@@ -2,6 +2,7 @@
 #include <set>
 #include <tuple>
 #include <sstream>
+#include <cassert>
 #include "machine.h"
 #include "parser.h"
 
@@ -9,39 +10,8 @@ namespace mixal {
 
 Machine::Machine() : rA(), rX(), rI1(), rI2(), rI3(), rI4(), rI5(), rI6(), rJ(),
       overflow(false), comparison(ComparisonIndicator::EQUAL), memory(),
-      devices(NUM_DEVICE, nullptr),
-      _pesudoVarIndex(), _lineOffset(), _elapsed(), _constants(), _lineNumbers() {}
-
-std::shared_ptr<IODevice> Machine::getDevice(int32_t index) {
-    if (devices[index] == nullptr) {
-        switch (index) {
-        case 0: case 1: case 2: case 3:
-        case 4: case 5: case 6: case 7:
-            devices[index] = std::shared_ptr<IODevice>(new IODeviceTape());
-            break;
-        case 8: case 9: case 10: case 11:
-        case 12: case 13: case 14: case 15:
-            devices[index] = std::shared_ptr<IODevice>(new IODeviceDisk());
-            break;
-        case 16:
-            devices[index] = std::shared_ptr<IODevice>(new IODeviceCardReader());
-            break;
-        case 17:
-            devices[index] = std::shared_ptr<IODevice>(new IODeviceCardPunch());
-            break;
-        case 18:
-            devices[index] = std::shared_ptr<IODevice>(new IODeviceLinePrinter());
-            break;
-        case 19:
-            devices[index] = std::shared_ptr<IODevice>(new IODeviceTypewriter());
-            break;
-        case 20:
-            devices[index] = std::shared_ptr<IODevice>(new IODevicePaperTape());
-            break;
-        }
-    }
-    return devices[index];
-}
+      devices(NUM_IO_DEVICE, nullptr),
+      _pesudoVarIndex(), _lineOffset(), _elapsed(), _constants() {}
 
 Register2& Machine::rI(int index) {
     switch (index) {
@@ -75,14 +45,13 @@ void Machine::reset() {
     for (int i = 0; i < NUM_MEMORY; ++i) {
         memory[i].reset();
     }
-    for (size_t i = 0; i < NUM_DEVICE; ++i) {
+    for (size_t i = 0; i < NUM_IO_DEVICE; ++i) {
         devices[i] = nullptr;
     }
     _pesudoVarIndex = 0;
     _lineOffset = 0;
     _elapsed = 0;
     _constants.clear();
-    _lineNumbers.clear();
 }
 
 std::string Machine::getSingleLineSymbol() {
@@ -107,6 +76,7 @@ void Machine::executeUntilSelfLoop() {
         }
         lastOffset = _lineOffset;
     }
+    waitDevices();
 }
 
 void Machine::executeUntilHalt() {
@@ -121,6 +91,7 @@ void Machine::executeUntilHalt() {
         }
         executeSingle(memory[_lineOffset]);
     }
+    waitDevices();
 }
 
 void Machine::executeSingle(ParsedResult* instruction) {
@@ -329,8 +300,6 @@ void Machine::executeSingle(const InstructionWord& instruction) {
     case Instructions::CMPX:
         executeCMP(instruction, &rX);
         break;
-    default:
-        break;
     }
     ++_lineOffset;
     _elapsed += Instructions::getCost(static_cast<Instructions::Code>(instruction.operation()),
@@ -348,8 +317,6 @@ void Machine::executeSinglePesudo(ParsedResult* instruction) {
     case Instructions::CON:
         executeCON(instruction);
         break;
-    default:
-        break;
     }
 }
 
@@ -384,15 +351,14 @@ void Machine::loadCodes(const std::vector<std::string>& codes, bool addHalt) {
                 if (!result.rawLocation.empty()) {
                     lineSymbol = result.rawLocation;
                 }
-                constants.push_back({lineSymbol,
-                                     Expression::getConstOffsetExpression(lineBase, lineOffset++),
-                                     result.address});
+                constants.push_back(std::make_tuple(
+                    lineSymbol,
+                    Expression::getConstOffsetExpression(lineBase, lineOffset++),
+                    result.address));
                 break;
             case Instructions::END:
                 endIndex = codeIndex;
                 break;
-            default:
-                break;
             }
         } else if (result.parsedType == ParsedType::INSTRUCTION) {
             bool usedLineSymbol = !result.rawLocation.empty() ||
@@ -438,18 +404,20 @@ void Machine::loadCodes(const std::vector<std::string>& codes, bool addHalt) {
     if (addHalt) {
         auto lineSymbol = getPesudoSymbolname();
         auto haltCommand = Parser::parseLine("HLT", lineSymbol, false);
-        constants.push_back({lineSymbol,
-                             Expression::getConstOffsetExpression(lineBase, lineOffset++),
-                             Expression::getConstExpression(AtomicValue(haltCommand.word.value()))});
+        constants.push_back(std::make_tuple(
+            lineSymbol,
+             Expression::getConstOffsetExpression(lineBase, lineOffset++),
+             Expression::getConstExpression(AtomicValue(haltCommand.word.value()))));
     }
     // Add expressions and literal constants
     for (auto& result : results) {
         if (!result.rawAddress.empty()) {
             if (result.address.literalConstant()) {
                 auto lineSymbol = getPesudoSymbolname();
-                constants.push_back({lineSymbol,
-                                    Expression::getConstOffsetExpression(lineBase, lineOffset++),
-                                    result.address});
+                constants.push_back(std::make_tuple(
+                    lineSymbol,
+                    Expression::getConstOffsetExpression(lineBase, lineOffset++),
+                    result.address));
                 result.address = Expression::getConstExpression(lineSymbol);
             }
             expressions[getPesudoSymbolname()] = &result.address;
@@ -463,9 +431,10 @@ void Machine::loadCodes(const std::vector<std::string>& codes, bool addHalt) {
     }
     if (endIndex != -1) {
         auto lineSymbol = results[endIndex].rawLocation.empty() ? getPesudoSymbolname() : results[endIndex].rawLocation;
-        constants.push_back({lineSymbol,
-                             Expression::getConstOffsetExpression(lineBase, lineOffset++),
-                             results[endIndex].address});
+        constants.push_back(std::make_tuple(
+            lineSymbol,
+            Expression::getConstOffsetExpression(lineBase, lineOffset++),
+            results[endIndex].address));
     }
     // Add collected constants to expressions
     for (auto& it : constants) {
@@ -502,21 +471,29 @@ void Machine::loadCodes(const std::vector<std::string>& codes, bool addHalt) {
     }
     // Load constants to memory
     for (auto& it : constants) {
-        memory[std::get<1>(it).result().value].set(std::get<2>(it).result().value);
+        int32_t location = std::get<1>(it).result().value;
+        if (location < 0 || location >= NUM_MEMORY) {
+            throw RuntimeError(location, "Location of the code is invalid: " + std::to_string(location));
+        }
+        memory[location].set(std::get<2>(it).result().value);
     }
     // Load results to memory
     _lineOffset = -1;
     for (auto& result : results) {
         if (result.parsedType == ParsedType::INSTRUCTION) {
             result.evaluate(evaluated);
-            if (_lineOffset == -1) {
-                _lineOffset = result.location.result().value;
+            int32_t location = result.location.result().value;
+            if (location < 0 || location >= NUM_MEMORY) {
+                throw RuntimeError(location, "Location of the code is invalid: " + std::to_string(location));
             }
-            memory[result.location.result().value].set(result.word.sign,
-                                                       result.word.address(),
-                                                       result.word.index(),
-                                                       result.word.field(),
-                                                       result.word.operation());
+            if (_lineOffset == -1) {
+                _lineOffset = location;
+            }
+            memory[location].set(result.word.negative,
+                                 result.word.address(),
+                                 result.word.index(),
+                                 result.word.field(),
+                                 result.word.operation());
         }
     }
     if (endIndex != -1) {
@@ -531,13 +508,18 @@ std::string Machine::getPesudoSymbolname() {
     return "#" + std::to_string(_pesudoVarIndex++);
 }
 
-int Machine::getIndexedAddress(const InstructionWord& instruction) {
-    int offset = 0;
+int32_t Machine::getIndexedAddress(const InstructionWord& instruction, bool checkRange) {
+    int32_t offset = 0;
     if (instruction.index() != 0) {
         auto& rIi = rI(instruction.index());
-        offset = static_cast<int>(rIi.value());
+        offset = static_cast<int32_t>(rIi.value());
     }
-    return static_cast<int>(instruction.addressValue()) + offset;
+    int32_t address = static_cast<int32_t>(instruction.addressValue()) + offset;
+    if (checkRange && !(0 <= address && address < NUM_MEMORY)) {
+        throw RuntimeError(_lineOffset, "Invalid address in instruction '" + instruction.getBytesString() +
+                                        "': " + std::to_string(address));
+    }
+    return address;
 }
 
 void Machine::copyToRegister5(const InstructionWord& instruction, const ComputerWord& word, Register5* reg) {
@@ -545,7 +527,7 @@ void Machine::copyToRegister5(const InstructionWord& instruction, const Computer
     int stop = instruction.field() % 8;
     reg->reset();
     if (start == 0) {
-        reg->sign = word.sign;
+        reg->negative = word.negative;
         ++start;
     }
     for (int i = stop, j = 5; i >= start; --i, --j) {
@@ -557,7 +539,7 @@ void Machine::copyFromRegister5(const InstructionWord& instruction, const Regist
     int start = instruction.field() / 8;
     int stop = instruction.field() % 8;
     if (start == 0) {
-        word->sign = reg.sign;
+        word->negative = reg.negative;
         ++start;
     }
     for (int i = stop, j = 5; i >= start; --i, --j) {
@@ -570,7 +552,7 @@ void Machine::copyToRegister2(const InstructionWord& instruction, const Computer
     int stop = instruction.field() % 8;
     reg->reset();
     if (start == 0) {
-        reg->sign = word.sign;
+        reg->negative = word.negative;
         ++start;
     }
     for (int i = stop, j = 2; i >= start && j > 0; --i, --j) {
@@ -578,9 +560,14 @@ void Machine::copyToRegister2(const InstructionWord& instruction, const Computer
     }
 }
 
+/** Check whether the given value can be fitted into the given number of bytes.
+ * 
+ * Overflow will be triggered if it the value can not be fitted into 5 bytes.
+ * (Which means rI will not trigger overflow.)
+ */
 int32_t Machine::checkRange(int32_t value, int bytes) {
     int32_t range = 1 << (6 * bytes);
-    if (abs(value) >= range) {
+    if (std::abs(value) >= range) {
         if (bytes == 5) {
             overflow = true;
         }
@@ -590,16 +577,12 @@ int32_t Machine::checkRange(int32_t value, int bytes) {
 }
 
 uint8_t Machine::getAX(int index) const {
-    if (index <= 0 || index > 10) {
-        throw std::runtime_error("Invalid index for AX: " + std::to_string(index));
-    }
+    assert(1 <= index && index <= 10);
     return index <= 5 ? rA[index] : rX[index - 5];
 }
 
 void Machine::setAX(int index, uint8_t value) {
-    if (index <= 0 || index > 10) {
-        throw std::runtime_error("Invalid index for AX: " + std::to_string(index));
-    }
+    assert(1 <= index && index <= 10);
     if (index <= 5) {
         rA[index] = value;
     } else {
